@@ -1,12 +1,16 @@
 package Hash::MultiValue;
 
 use strict;
+no warnings 'void';
 use 5.006_002;
-our $VERSION = '0.11';
+our $VERSION = '0.13';
 
 use Carp ();
 use Scalar::Util qw(refaddr);
 
+# there does not seem to be a relevant RT or perldelta entry for this
+use constant _SPLICE_SAME_ARRAY_SEGFAULT => $] < '5.008007';
+
 my %keys;
 my %values;
 my %registry;
@@ -39,9 +43,7 @@ sub create {
     my $this = refaddr $self;
     $keys{$this} = [];
     $values{$this} = [];
-    if (NEEDS_REGISTRY) {
-        Scalar::Util::weaken($registry{$this} = $self);
-    }
+    Scalar::Util::weaken($registry{$this} = $self) if NEEDS_REGISTRY;
     $self;
 }
 
@@ -63,9 +65,7 @@ sub DESTROY {
     my $this = refaddr shift;
     delete $keys{$this};
     delete $values{$this};
-    if (NEEDS_REGISTRY) {
-        delete $registry{$this};
-    }
+    delete $registry{$this} if NEEDS_REGISTRY;
 }
 
 sub get {
@@ -107,21 +107,17 @@ sub set {
     elsif ($added < 0) {
         my ($start, @drop, @keep) = splice @idx, $added;
         for my $i ($start+1 .. $#$k) {
-            if ($i == $drop[0]) {
-              shift @drop;
-              next;
+            if (@drop and $i == $drop[0]) {
+                shift @drop;
+                next;
             }
             push @keep, $i;
         }
 
-        # this used to be written as
-        #   splice @$_, $start, 0+@$_, @$_[@keep]
-        # however older perls crash on attempts to splice-replace a subscript
-        # of the array currently being splice()d
-        #
-        # I can not seem to find a relevant RT or perldelta entry, but this
-        # seems to have been fixed in 5.8.7
-        @$_ = @$_[0 .. $start-1, @keep] for ($k, $v);
+        splice @$_, $start, 0+@$_, ( _SPLICE_SAME_ARRAY_SEGFAULT
+            ? @{[ @$_[@keep] ]} # force different source array
+            :     @$_[@keep]
+        ) for $k, $v;
     }
 
     if (@_) {
@@ -258,6 +254,22 @@ sub as_hashref_multi {
 
 sub multi { $_[0]->as_hashref_multi }
 
+sub STORABLE_freeze {
+    my $self = shift;
+    my $this = refaddr $self;
+    return '', $keys{$this}, $values{$this};
+}
+
+sub STORABLE_thaw {
+    my $self = shift;
+    my ($is_cloning, $serialised, $k, $v) = @_;
+    my $this = refaddr $self;
+    $keys  {$this} = $k;
+    $values{$this} = $v;
+    @{$self}{@$k} = @$v;
+    return $self;
+}
+
 1;
 __END__
 
